@@ -9,13 +9,74 @@ import functools
 import numbers
 import sys
 from dataclasses import dataclass
-from itertools import chain
+from typing import (
+    Iterator,
+    List,
+    Optional,
+)
 
 __author__ = "Nicola Coretti"
 __email__ = "nico.coretti@gmail.com"
 
 
-class AbstractCrcRegister(metaclass=abc.ABCMeta):
+class Byte(numbers.Number):
+    BIT_LENGTH: int = 8
+    BIT_MASK: int = 0xFF
+
+    def __init__(self, value: int = 0x00):
+        self._value = value & Byte.BIT_MASK
+
+    def __add__(self, other: "Byte") -> "Byte":
+        if not isinstance(other, Byte):
+            other = Byte(other)
+        return Byte(self.value + other.value)
+
+    def __radd__(self, other: "Byte") -> "Byte":
+        return self + other
+
+    def __iadd__(self, other: "Byte") -> "Byte":
+        result = self + other
+        self.value = result.value
+        return self
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Byte):
+            return False
+        return self.value == other.value
+
+    def __hash__(self) -> int:
+        return hash(self.value)
+
+    def __len__(self) -> int:
+        return Byte.BIT_LENGTH
+
+    def __getitem__(self, index: int) -> int:
+        if index >= Byte.BIT_LENGTH or index < 0:
+            raise IndexError
+        return (self.value & (1 << index)) >> index
+
+    def __iter__(self) -> Iterator[int]:
+        return (self[i] for i in range(0, len(self)))
+
+    def __int__(self):
+        return self.value
+
+    @property
+    def value(self) -> int:
+        return self._value & Byte.BIT_MASK
+
+    @value.setter
+    def value(self, value) -> None:
+        self._value = value & Byte.BIT_MASK
+
+    def reversed(self) -> "Byte":
+        value = 0
+        for index, bit in enumerate(reversed(self)):
+            value += bit << index
+        return Byte(value)
+
+
+class AbstractRegister(metaclass=abc.ABCMeta):
     """
     Abstract base class / Interface a crc register needs to implement.
 
@@ -32,30 +93,33 @@ class AbstractCrcRegister(metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def update(self, data):
+    def update(self, data: bytes) -> int:
         """
-        Feeds the provided data into the crc register.
+        Feeds data into the register.
 
-        :param bytes data: a bytes like object or ann object which can be converted to a bytes
-                     like object using the built in bytes() function.
-        :return: the current value of the crc register.
+        Args:
+            data: which will be feed into the register.
+
+        Returns:
+            Register content after the update.
         """
 
     @abc.abstractmethod
-    def digest(self):
+    def digest(self) -> int:
         """
         Final crc checksum will be calculated.
 
-        :return: the final crc checksum.
-        :rtype: int.
+        Returns:
+            Final crc result/value (applies pending operations like final xor).
         """
 
     @abc.abstractmethod
-    def reverse(self):
+    def reverse(self) -> int:
         """
         Calculates the reversed value of the crc register.
 
-        :return: the the reversed value of the crc register.
+        Returns:
+            The reversed value of the crc register.
         """
 
 
@@ -74,17 +138,18 @@ class Configuration:
     reverse_output: bool = False
 
 
-class CrcRegisterBase(AbstractCrcRegister):
+class BasicRegister(AbstractRegister):
     """
     Implements the common crc algorithm, assuming a user of this base
     class will provide an overwrite for the _process_byte method.
     """
 
-    def __init__(self, configuration):
+    def __init__(self, configuration: Configuration):
         """
-        Create a new CrcRegisterBase.
+        Create a new BasicRegister.
 
-        :param configuration: used for the crc algorithm.
+        Args:
+            configuration: Used to configure the crc algorithm.
         """
         if isinstance(configuration, enum.Enum):
             configuration = configuration.value
@@ -93,64 +158,71 @@ class CrcRegisterBase(AbstractCrcRegister):
         self._config = configuration
         self._register = configuration.init_value & self._bitmask
 
-    def __len__(self):
+    def __len__(self) -> int:
         """
-        Returns the length (width) of the register.
-
-        :return: the register size/width in bytes.
+        Returns:
+            The width of the register.
         """
         return self._config.width // 8
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> int:
         """
         Gets a single byte of the register.
 
-        :param index: byte which shall be returned.
-        :return: the byte at the specified index.
-        :raises IndexError: if the index is out of bounce.
+        Args:
+            index: byte which shall be returned.
+
+        Returns:
+            The byte at the specified index.
+
+        Raises:
+            IndexError: Invalid index for this register.
         """
         if index >= (self._config.width / 8) or index < 0:
             raise IndexError
         shift_offset = index * 8
         return (self.register & (0xFF << shift_offset)) >> shift_offset
 
-    def init(self):
+    def init(self) -> None:
         """
-        See AbstractCrcRegister.init
+        See AbstractRegister.init
         """
         self.register = self._config.init_value
 
-    def update(self, data):
+    def update(self, data: bytes) -> int:
         """
-        See AbstractCrcRegister.update
+        See AbstractRegister.update
         """
-        for byte in data:
-            byte = Byte(byte)
+        for byte in (Byte(b) for b in data):
             if self._config.reverse_input:
                 byte = byte.reversed()
             self._register = self._process_byte(byte)
         return self.register
 
     @abc.abstractmethod
-    def _process_byte(self, byte):
+    def _process_byte(self, byte: Byte) -> int:
         """
-        Processes an entire byte feed to the crc register.
+        Feed a byte into the crc register.
 
-        :param byte: the byte which shall be processed by the crc register.
-        :return: the new value of the crc register will have after the byte have been processed.
+        Args:
+            byte: the byte which shall be processed by the crc register.
+
+        Returns:
+            The value/state the crc register needs to be put in
+            after this byte has been processed.
         """
 
-    def digest(self):
+    def digest(self) -> int:
         """
-        See AbstractCrcRegister.digest
+        See AbstractRegister.digest
         """
         if self._config.reverse_output:
             self.register = self.reverse()
         return self.register ^ self._config.final_xor_value
 
-    def reverse(self):
+    def reverse(self) -> int:
         """
-        See AbstractCrcRegister.digest
+        See AbstractRegister.digest
         """
         index = 0
         reversed_value = 0
@@ -159,19 +231,19 @@ class CrcRegisterBase(AbstractCrcRegister):
             index += 8
         return reversed_value
 
-    def _is_division_possible(self):
+    def _is_division_possible(self) -> bool:
         return (self.register & self._topbit) > 0
 
     @property
-    def register(self):
+    def register(self) -> int:
         return self._register & self._bitmask
 
     @register.setter
-    def register(self, value):
+    def register(self, value) -> None:
         self._register = value & self._bitmask
 
 
-class CrcRegister(CrcRegisterBase):
+class Register(BasicRegister):
     """
     Simple crc register, which will process one bit at the time.
 
@@ -181,9 +253,9 @@ class CrcRegister(CrcRegisterBase):
         based register.
     """
 
-    def _process_byte(self, byte):
+    def _process_byte(self, byte: Byte) -> int:
         """
-        See CrcRegisterBase._process_byte
+        See BasicRegister._process_byte
         """
         self.register ^= int(byte) << (self._config.width - 8)
         for _ in byte:
@@ -194,24 +266,27 @@ class CrcRegister(CrcRegisterBase):
         return self.register
 
 
-class TableBasedCrcRegister(CrcRegisterBase):
+class TableBasedRegister(BasicRegister):
     """
     Lookup table based crc register.
 
     .. note::
 
-        this register type will be much faster than a simple bit by bit based crc register.
-        (e.g. CrcRegister)
+        this register type will be much faster than a simple bit
+        by bit based crc register (e.g. Register).
     """
 
-    def __init__(self, configuration):
+    def __init__(self, configuration: Configuration):
         """
         Creates a new table based crc register.
 
-        :param configuration: used for the crc algorithm.
+        Args:
+            configuration: used for the crc algorithm.
 
-        :attention: creating a table based register initially might take some extra time, due to the
-                    fact that some lookup tables need to be calculated/initialized .
+        :attention:
+
+            creating a table based register initially might take some extra time,
+            due to the fact that some lookup tables need to be calculated/initialized .
         """
         super().__init__(configuration)
         if isinstance(configuration, enum.Enum):
@@ -220,113 +295,64 @@ class TableBasedCrcRegister(CrcRegisterBase):
             configuration.width, configuration.polynomial
         )
 
-    def _process_byte(self, byte):
+    def _process_byte(self, byte: Byte) -> int:
         """
-        See CrcRegisterBase._process_byte
+        See BasicRegister._process_byte
         """
         index = int(byte) ^ (self.register >> (self._config.width - 8))
         self.register = self._lookup_table[index] ^ (self.register << 8)
         return self.register
 
 
-class Byte(numbers.Number):
-    BIT_LENGTH = 8
-    BIT_MASK = 0xFF
-
-    def __init__(self, value=0x00):
-        self._value = value & Byte.BIT_MASK
-
-    def __add__(self, other):
-        if not isinstance(other, Byte):
-            other = Byte(other)
-        return Byte(self.value + other.value)
-
-    def __radd__(self, other):
-        return self + other
-
-    def __iadd__(self, other):
-        result = self + other
-        self.value = result.value
-        return self
-
-    def __eq__(self, other):
-        if not isinstance(other, Byte):
-            raise TypeError("unsupported operand")
-        return self.value == other.value
-
-    def __hash__(self):
-        return hash(self.value)
-
-    def __len__(self):
-        return Byte.BIT_LENGTH
-
-    def __getitem__(self, index):
-        if index >= Byte.BIT_LENGTH or index < 0:
-            raise IndexError
-        return (self.value & (1 << index)) >> index
-
-    def __int__(self):
-        return self.value
-
-    @property
-    def value(self):
-        return self._value & Byte.BIT_MASK
-
-    @value.setter
-    def value(self, value):
-        self._value = value & Byte.BIT_MASK
-
-    def reversed(self):
-        value = 0
-        index = 0
-        for bit in reversed(self):
-            value += bit << index
-            index += 1
-        return Byte(value)
-
-
 @functools.lru_cache()
-def create_lookup_table(width, polynomial):
+def create_lookup_table(width: int, polynomial: int) -> List[int]:
     """
     Creates a crc lookup table.
 
-    :param int width: of the crc checksum.
-    :parma int polynomial: which is used for the crc calculation.
+    Args:
+        width: of the crc checksum.
+        polynomial: which is used for the crc calculation.
+
+    Returns:
+        The lookup table for the specified width and polynomial.
     """
     config = Configuration(width=width, polynomial=polynomial)
-    crc_register = CrcRegister(config)
+    crc_register = Register(config)
     lookup_table = []
     for index in range(0, 256):
         crc_register.init()
-        data = bytes((index).to_bytes(1, byteorder="big"))
+        data = bytes(index.to_bytes(1, byteorder="big"))
         crc_register.update(data)
         lookup_table.append(crc_register.digest())
     return lookup_table
 
 
-class CrcCalculator:
-    def __init__(self, configuration, table_based=False):
+class Calculator:
+    def __init__(self, configuration: Configuration, optimized: bool = False):
         """
-        Creates a new CrcCalculator.
+        Creates a new Calculator.
 
-        :param configuration: for the crc algorithm.
-        :param table_based: if true a tables based register will be used for the calculations.
+        Args:
+            configuration: for the crc algorithm.
+            optimized: whether a register optimized for speed shall be used.
 
-        :attention: initializing a table based calculator might take some extra time, due to the
-                    fact that the lookup table need to be initialized.
+        :attention: initializing an optimized calculator might take some extra time,
+                    calculation itself will be faster though.
         """
-        if table_based:
-            self._crc_register = TableBasedCrcRegister(configuration)
-        else:
-            self._crc_register = CrcRegister(configuration)
+        _types = {
+            False: Register,
+            True: TableBasedRegister,
+        }
+        klass = _types[optimized]
+        self._crc_register = klass(configuration)
 
-    def calculate_checksum(self, data):
+    def checksum(self, data: bytes) -> int:
         self._crc_register.init()
         self._crc_register.update(data)
         return self._crc_register.digest()
 
-    def verify_checksum(self, data, expected_checksum):
-        return self.calculate_checksum(data) == expected_checksum
+    def verify(self, data: bytes, expected_checksum: int) -> bool:
+        return self.checksum(data) == expected_checksum
 
 
 @enum.unique
@@ -458,15 +484,7 @@ class Crc64(enum.Enum):
     )
 
 
-CRC_TYPES = {
-    Crc8.__name__: Crc8,
-    Crc16.__name__: Crc16,
-    Crc32.__name__: Crc32,
-    Crc64.__name__: Crc64,
-}
-
-
-def argument_parser():
+def _argument_parser() -> argparse.ArgumentParser:
     into_int = functools.partial(int, base=0)
     program = "crc"
     description = "A set of crc checksum related command line tools."
@@ -494,34 +512,14 @@ def argument_parser():
         help="hex value of the polynomial used for calculating the crc table",
     )
     table_command.set_defaults(func=table)
-
-    checksum_command = subparsers.add_parser(
-        "checksum", help="Calculate checksum(s) for the specified input(s)"
-    )
-    checksum_command.add_argument(
-        "inputs",
-        nargs="*",
-        type=argparse.FileType("rb"),
-        default=[sys.stdin.buffer],
-        help="who will be feed into the crc calculation",
-    )
-    checksum_command.add_argument(
-        "-c",
-        "--category",
-        choices=list(CRC_TYPES),
-        default=Crc8.__name__,
-        help="of crc algorithms which shall be used for calculation",
-    )
-    checksum_command.set_defaults(func=checksum)
-
     return parser
 
 
-def _generate_template(width):
+def _generate_template(width: int) -> str:
     return f"0x{{:0{(width + 3) // 4}X}}"
 
 
-def table(args):
+def table(args: argparse.Namespace) -> bool:
     if not (args.width and args.polynomial):
         return False
     columns = 8
@@ -530,25 +528,12 @@ def table(args):
     lookup_table = create_lookup_table(width, polynomial)
     template = _generate_template(width)
     rows = (lookup_table[i : i + columns] for i in range(0, len(lookup_table), columns))
-    print("\n".join((" ".join((template.format(value) for value in r)) for r in rows)))
+    print("\n".join(" ".join(template.format(value) for value in r) for r in rows))
     return True
 
 
-def checksum(args):
-    category = CRC_TYPES[args.category]
-    data = bytearray(chain.from_iterable(src.read() for src in args.inputs))
-    for algorithm in sorted(category, key=str):
-        print(
-            "{name}: 0x{result:X}".format(
-                name=f"{algorithm}".split(".")[1],
-                result=CrcCalculator(algorithm, True).calculate_checksum(data),
-            )
-        )
-    return True
-
-
-def main(argv=None):
-    parser = argument_parser()
+def main(argv: Optional[List[str]] = None):
+    parser = _argument_parser()
     args = parser.parse_args(argv)
     if "func" in args:
         exit_code = 0 if args.func(args) else -1
